<?php
use Automattic\WooCommerce\Blocks\Integrations\IntegrationInterface;

	/**
	 * Class for integrating with WooCommerce Blocks
	 */
class Af_Wc_Daps_Checkout_Blocks_Integration implements IntegrationInterface {

	/**
	 * The name of the integration.
	 *
	 * @return string
	 */
	public function get_name() {
		return 'af_wc_daps_checkout_block';
	}

	/**
	 * When called invokes any initialization/setup for the integration.
	 */
	public function initialize() {

		$this->register_af_wc_daps_block__block_frontend_scripts();
		$this->register_af_wc_daps_block__block_editor_scripts();
		$this->register_main_integration();

		add_action( 'wp_enqueue_scripts', array( $this, 'af_wc_daps_enqueue_front_script' ) );      
	}

	public function af_wc_daps_enqueue_front_script() {

		wp_enqueue_script( 'af-wc-daps-front-script', plugins_url( '/src/js/af-wc-daps-checkout-block/block.js', __FILE__ ), '', '1.0', false );
		wp_enqueue_script( 'jquery' );

		$applied_rule_settings = $this->af_wc_daps_display_delivery_info_form_on_checkout_page_for_blocks();

		$rule_applied = '';
		$rule_data    = array();


		//for expected date
		if (isset($applied_rule_settings['selection_mode']) && 'expected_date' == $applied_rule_settings['selection_mode']) {
			
			
			$rule_data = array(
				'selection_mode' => 'expected_date',
				'rule_data'      => $applied_rule_settings['rule_data'],
			);
	
			WC()->session->set( 'af_wc_daps_rule_data', $rule_data );

			$rule_applied = 'expected_date';

		} elseif (isset($applied_rule_settings['selection_mode']) && 'calendar' == $applied_rule_settings['selection_mode']) {//for calendar
			
			
			$rule_data = array(
				'selection_mode' => 'calendar',
				'rule_data'      => $applied_rule_settings['rule_data'],
			);
	
			$rule_applied = 'calendar';

		}



		$af_wc_daps_expected_info_style_settings = (array) get_option( 'af_wc_daps_expected_info_style_settings' );

		$enable_icon = isset( $af_wc_daps_expected_info_style_settings['af_wc_daps_enable_icon_for_expected_date_info'] ) ? $af_wc_daps_expected_info_style_settings['af_wc_daps_enable_icon_for_expected_date_info'] : '';
		$icon_url    = isset( $af_wc_daps_expected_info_style_settings['af_wc_daps_icon_for_expected_date_info'] ) ? $af_wc_daps_expected_info_style_settings['af_wc_daps_icon_for_expected_date_info'] : '';
		$image_width = isset( $af_wc_daps_expected_info_style_settings['af_wc_daps_icon_width_expected_date_info'] ) && '' != $af_wc_daps_expected_info_style_settings['af_wc_daps_icon_width_expected_date_info'] ? $af_wc_daps_expected_info_style_settings['af_wc_daps_icon_width_expected_date_info'] : '42';
		$bg_color    = isset( $af_wc_daps_expected_info_style_settings['af_wc_daps_bg_color_of_expected_date_container'] ) ? $af_wc_daps_expected_info_style_settings['af_wc_daps_bg_color_of_expected_date_container'] : '#EFF3F5';
		$text_color  = isset( $af_wc_daps_expected_info_style_settings['af_wc_daps_text_color_of_expected_date_container'] ) ? $af_wc_daps_expected_info_style_settings['af_wc_daps_text_color_of_expected_date_container'] : '#6d6d6d';

		$af_wc_daps_general_setting = get_option('af_wc_daps_general_settings');

		$af_wc_daps_delivery_info_field_position   = isset( $af_wc_daps_general_setting['af_wc_daps_delivery_info_field_position'])?$af_wc_daps_general_setting['af_wc_daps_delivery_info_field_position']:'woocommerce_before_checkout_billing_form';
		$af_wc_daps_mandatory_order_delivery_field = isset( $af_wc_daps_general_setting['af_wc_daps_mandatory_order_delivery_field'] ) ? true : false;
		$af_wc_daps_mandatory_order_timeslot_field = isset( $af_wc_daps_general_setting['af_wc_daps_mandatory_order_timeslot_field'] ) ? true : false;

		$af_wc_daps_label_settings = get_option('af_wc_daps_label_settings');

		$af_wc_daps_order_delivery_date_msg = isset($af_wc_daps_label_settings['af_wc_daps_order_delivery_date_msg']) && '' != $af_wc_daps_label_settings['af_wc_daps_order_delivery_date_msg']?$af_wc_daps_label_settings['af_wc_daps_order_delivery_date_msg']:'Please select a delivery date' ;      
		$af_wc_daps_order_timeslot_msg      = isset($af_wc_daps_label_settings['af_wc_daps_order_timeslot_msg']) && '' != $af_wc_daps_label_settings['af_wc_daps_order_timeslot_msg']?$af_wc_daps_label_settings['af_wc_daps_order_timeslot_msg']:'Please select a time slot' ;
			


		$general_settings = array(
			'applied_rule'                            => $rule_applied,
			'enable_icon'                             => $enable_icon,
			'icon_url'                                => $icon_url,
			'image_width'                             => $image_width,
			'bg_color'                                => $bg_color,
			'text_color'                              => $text_color,
			'base_url'                                => AF_WC_DAPS_PLUGIN_URL,
			'af_wc_daps_delivery_info_field_position' => $af_wc_daps_delivery_info_field_position,
			'is_mandatory_order_delivery_field'       => $af_wc_daps_mandatory_order_delivery_field,
			'is_mandatory_order_timeslot_field'       => $af_wc_daps_mandatory_order_timeslot_field,
			'af_wc_daps_order_delivery_date_msg'      => $af_wc_daps_order_delivery_date_msg,
			'af_wc_daps_order_timeslot_msg'           => $af_wc_daps_order_timeslot_msg,
		);
			
		wp_localize_script(
			'af-wc-daps-front-script',
			'af_wc_daps_blocks_data',
			array_merge($rule_data, $general_settings)
		);
	}


	/**
	 * Registers the main JS file required to add filters and Slot/Fills.
	 */
	public function register_main_integration() {
		$script_path       = '/build/index.js';
		$script_url        = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/index.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_path ),
			);

		wp_register_script(
			'af-wc-daps-checkout-blocks-integration',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);
		wp_set_script_translations(
			'af-wc-daps-checkout-blocks-integration',
			'af_wc_daps_checkout_block',
			__DIR__ . '/languages'
		);
	}

	/**
	 * Returns an array of script handles to enqueue in the frontend context.
	 *
	 * @return string[]
	 */
	public function get_script_handles() {
		return array( 'af-wc-daps-checkout-blocks-integration', 'af-wc-daps-checkout-block-frontend' );
	}

	/**
	 * Returns an array of script handles to enqueue in the editor context.
	 *
	 * @return string[]
	 */
	public function get_editor_script_handles() {
		return array( 'af-wc-daps-checkout-blocks-integration', 'af-wc-daps-checkout-block-editor' );
	}

	/**
	 * An array of key, value pairs of data made available to the block on the client side.
	 *
	 * @return array
	 */
	public function get_script_data() {
	}


	public function register_af_wc_daps_block__block_editor_scripts() {
		$script_path       = '/build/af-wc-daps-checkout-block.js';
		$script_url        = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/af-wc-daps-checkout-block.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_asset_path ),
			);

		wp_register_script(
			'af-wc-daps-checkout-block-editor',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);

		wp_set_script_translations(
			'af-wc-daps-checkout-block-editor',
			'af_wc_daps_checkout_block',
			__DIR__ . '/languages'
		);
	}



	public function register_af_wc_daps_block__block_frontend_scripts() {
		$script_path       = '/build/af-wc-daps-checkout-block-frontend.js';
		$script_url        = plugins_url( $script_path, __FILE__ );
		$script_asset_path = __DIR__ . '/build/af-wc-daps-checkout-block-frontend.asset.php';
		$script_asset      = file_exists( $script_asset_path )
			? require $script_asset_path
			: array(
				'dependencies' => array(),
				'version'      => $this->get_file_version( $script_asset_path ),
			);

		wp_register_script(
			'af-wc-daps-checkout-block-frontend',
			$script_url,
			$script_asset['dependencies'],
			$script_asset['version'],
			true
		);
		wp_set_script_translations(
			'af-wc-daps-checkout-block-frontend',
			'af_wc_daps_checkout_block',
			__DIR__ . '/languages'
		);
	}

	/**
	 * Get the file modified time as a cache buster if we're in dev mode.
	 *
	 * @param string $file Local path to the file.
	 * @return string The cache buster value to use for the given file.
	 */
	protected function get_file_version( $file ) {
		if ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG && file_exists( $file ) ) {
			return filemtime( $file );
		}
		return AF_WC_DAPS_BLOCK_VERSION;
	}

	public function af_wc_daps_get_all_rules() {
		return get_posts(
			array(
				'post_type'      => 'daps_rules',
				'posts_per_page' => -1,
				'post_status'    => 'publish',
				'fields'         => 'ids',
				'orderby'        => 'menu_order',
				'order'          => 'DESC',
			)
		);
	}


	public function af_wc_daps_display_delivery_info_form_on_checkout_page_for_blocks() {

		$af_wc_daps_general_settings = (array) get_option( 'af_wc_daps_general_settings' );
			
		$af_wc_daps_enable_at_checkout_page = isset( $af_wc_daps_general_settings['af_wc_daps_enable_at_checkout_page'] ) ? $af_wc_daps_general_settings['af_wc_daps_enable_at_checkout_page'] : '';

		//order delivery is not enabled
		if ('1' != $af_wc_daps_enable_at_checkout_page) {
			return array();
		}

		global $woocommerce;

		$af_wc_daps_found_product_without_meta = false;

		$af_wc_daps_products_in_cart_on_which_rule_not_applied = array();

		$matched = false;
		foreach ( $woocommerce->cart->get_cart() as $cart_item_key => $cart_item ) {
			if ($cart_item['product_id']) {
				$af_wc_daps_rule_settings = (array) get_post_meta( $cart_item['product_id'], 'af_wc_daps_rule_settings', true );

				if ( isset( $af_wc_daps_rule_settings['af_wc_daps_enable_rule'] ) && 'yes' == $af_wc_daps_rule_settings['af_wc_daps_enable_rule'] ) {
					$matched = true;
					break;
				}

			}
		}

		if (empty($this->af_wc_daps_get_all_rules()) && !$matched) {
			return array();
		}

		foreach ( $woocommerce->cart->get_cart() as $cart_item_key => $cart_item ) {

			$af_wc_daps_product_delivery_date = isset( $cart_item['af_wc_daps_order_data']['af_wc_daps_single_product_page_date'] ) ? $cart_item['af_wc_daps_order_data']['af_wc_daps_single_product_page_date'] : '';
			$af_wc_daps_product_expected_date = isset( $cart_item['af_wc_daps_expected_date']['af_wc_daps_expected_date_info'] ) ? $cart_item['af_wc_daps_expected_date']['af_wc_daps_expected_date_info'] : '';

			if ( '' == $af_wc_daps_product_delivery_date && '' == $af_wc_daps_product_expected_date ) {

				$af_wc_daps_product_id = $cart_item['product_id'];

				if ( 0 != $cart_item['variation_id'] ) {
					$af_wc_daps_product_id = $cart_item['variation_id'];
				}

				$product = wc_get_product($af_wc_daps_product_id);

				//if product is virtual/downloadable skip it.
				if ($product->is_downloadable() || $product->is_virtual()) {
					continue;
				}

				$af_wc_daps_products_in_cart_on_which_rule_not_applied[] = $af_wc_daps_product_id;
				$af_wc_daps_found_product_without_meta                   = true;
			}
		}

		
		//apply the rule of that product which has the late date among other product
		$af_wc_daps_largest_minimum_days                      = 0; 
		$af_wc_daps_product_whose_rule_found_on_product_level = array();
		$af_wc_daps_product_id_with_largest_value             = isset($af_wc_daps_products_in_cart_on_which_rule_not_applied[0])?$af_wc_daps_products_in_cart_on_which_rule_not_applied[0]:'';

		foreach ($af_wc_daps_products_in_cart_on_which_rule_not_applied as $product_id) {
			$af_wc_daps_rule_settings = (array) get_post_meta($product_id, 'af_wc_daps_rule_settings', true);
	
			if ( empty( $af_wc_daps_rule_settings ) || !isset( $af_wc_daps_rule_settings['af_wc_daps_enable_rule'] ) ||  '' == $af_wc_daps_rule_settings['af_wc_daps_enable_rule'] ) {
				continue;
			}

			if ('calendar' == $af_wc_daps_rule_settings['af_wc_daps_rule_delivery_selection_mode']) {
				$af_wc_daps_minimum_days                                = isset($af_wc_daps_rule_settings['af_wc_daps_rule_min_available_days']) ? $af_wc_daps_rule_settings['af_wc_daps_rule_min_available_days'] : 1;
				$af_wc_daps_product_whose_rule_found_on_product_level[] =$product_id;
			} else {
				$af_wc_daps_minimum_days                                = isset($af_wc_daps_rule_settings['af_wc_daps_rule_minimum_delivery_days']) && '' != $af_wc_daps_rule_settings['af_wc_daps_rule_minimum_delivery_days'] ? $af_wc_daps_rule_settings['af_wc_daps_rule_minimum_delivery_days'] : 1;
				$af_wc_daps_product_whose_rule_found_on_product_level[] =$product_id;
			}

			if ($af_wc_daps_minimum_days > $af_wc_daps_largest_minimum_days) {
				$af_wc_daps_largest_minimum_days          = $af_wc_daps_minimum_days;
				$af_wc_daps_product_id_with_largest_value = $product_id;
			}
		}
		

		$af_wc_daps_rule_posts = $this->af_wc_daps_get_all_rules();
		$product_categories    = af_wc_daps_get_all_categories_for_product_ids_array( $af_wc_daps_products_in_cart_on_which_rule_not_applied );

		$af_wc_daps_largest_minimum_days_rule_level = 0; 
		$af_wc_daps_rule_id_with_largest_value      = null;
		
		if ($af_wc_daps_found_product_without_meta) {
			foreach ($af_wc_daps_products_in_cart_on_which_rule_not_applied as $product_id) {

				

				foreach ($af_wc_daps_rule_posts as $rule_id) {
					$is_product_match =false;
					if ( empty( $rule_id ) ) {
						continue;
					}
					if (in_array($product_id, $af_wc_daps_product_whose_rule_found_on_product_level)) {
						continue;
					}


					$af_wc_daps_post_level_settings   = (array) get_post_meta( $rule_id, 'af_wc_daps_rule_settings', true );
					$af_wc_daps_product_ids_for_rule  = isset( $af_wc_daps_post_level_settings['af_wc_daps_selected_product_ids'] ) ? $af_wc_daps_post_level_settings['af_wc_daps_selected_product_ids'] : array();
					$af_wc_daps_selected_category_ids = isset( $af_wc_daps_post_level_settings ['af_wc_daps_selected_category_ids'] ) ? $af_wc_daps_post_level_settings ['af_wc_daps_selected_category_ids'] : array();

					if ( empty( $af_wc_daps_post_level_settings ) || !isset( $af_wc_daps_post_level_settings['af_wc_daps_enable_rule'] ) ||  '' == $af_wc_daps_post_level_settings['af_wc_daps_enable_rule'] ) {
						continue;
					}
					

					if ( empty( $af_wc_daps_product_ids_for_rule ) && empty( $af_wc_daps_selected_category_ids ) ) {
						$is_product_match = true;
					} elseif ( ! empty( $af_wc_daps_product_ids_for_rule ) && in_array( $product_id, $af_wc_daps_product_ids_for_rule ) ) {
						$is_product_match = true;
					} elseif ( ! empty( $af_wc_daps_selected_category_ids ) && ! empty( array_intersect( $product_categories, $af_wc_daps_selected_category_ids ) ) ) {
						$is_product_match = true;
					}

					if ($is_product_match) {
						if ('calendar' == $af_wc_daps_post_level_settings['af_wc_daps_rule_delivery_selection_mode']) {
							$af_wc_daps_minimum_days                                = isset($af_wc_daps_post_level_settings['af_wc_daps_rule_min_available_days']) && ''!=$af_wc_daps_post_level_settings['af_wc_daps_rule_min_available_days'] ? $af_wc_daps_post_level_settings['af_wc_daps_rule_min_available_days'] : 1;
							$af_wc_daps_product_whose_rule_found_on_product_level[] =$product_id;
						} else {
							$af_wc_daps_minimum_days                                = isset($af_wc_daps_post_level_settings['af_wc_daps_rule_minimum_delivery_days']) &&  '' != $af_wc_daps_post_level_settings['af_wc_daps_rule_minimum_delivery_days']? $af_wc_daps_post_level_settings['af_wc_daps_rule_minimum_delivery_days'] : 1;
							$af_wc_daps_product_whose_rule_found_on_product_level[] =$product_id;
						}
						
						if ($af_wc_daps_minimum_days > $af_wc_daps_largest_minimum_days_rule_level) {
							$af_wc_daps_largest_minimum_days_rule_level = $af_wc_daps_minimum_days;
							$af_wc_daps_rule_id_with_largest_value      = $rule_id;
						}
					}

				}
			}
		}
		
		
		if ($af_wc_daps_product_id_with_largest_value && $af_wc_daps_largest_minimum_days >= $af_wc_daps_largest_minimum_days_rule_level) {
			$af_wc_daps_rule_settings = (array) get_post_meta( $af_wc_daps_product_id_with_largest_value, 'af_wc_daps_rule_settings', true );

			if ( ! empty( $af_wc_daps_rule_settings ) && isset( $af_wc_daps_rule_settings['af_wc_daps_enable_rule'] ) && 'yes' == $af_wc_daps_rule_settings['af_wc_daps_enable_rule'] ) {

				if ( isset( $af_wc_daps_rule_settings['af_wc_daps_rule_delivery_selection_mode'] ) && 'expected_date' == $af_wc_daps_rule_settings['af_wc_daps_rule_delivery_selection_mode'] ) {
					$af_wc_daps_rule_minimum_delivery_days      = isset( $af_wc_daps_rule_settings['af_wc_daps_rule_minimum_delivery_days'] ) ? $af_wc_daps_rule_settings['af_wc_daps_rule_minimum_delivery_days'] : '';
					$af_wc_daps_rule_maximum_delivery_days      = isset( $af_wc_daps_rule_settings['af_wc_daps_rule_maximum_delivery_days'] ) ? $af_wc_daps_rule_settings['af_wc_daps_rule_maximum_delivery_days'] : '';
					$af_wc_daps_rule_expected_date_message_text = isset( $af_wc_daps_rule_settings['af_wc_daps_rule_expected_date_message_text'] ) ? $af_wc_daps_rule_settings['af_wc_daps_rule_expected_date_message_text'] : '';
					return array(
						'selection_mode' => 'expected_date',
						'rule_data'      =>  af_wc_daps_expected_date_info_checkout_page_for_block( $af_wc_daps_rule_minimum_delivery_days, $af_wc_daps_rule_maximum_delivery_days, $af_wc_daps_rule_expected_date_message_text ),
					);
				}

				wp_localize_script(
				'af-wc-daps-front-script',
				'af_wc_daps_checkout_page_rule_settings',
				$af_wc_daps_rule_settings
				);
				return array(
					'selection_mode' => 'calendar',
					'rule_data'      =>  af_wc_daps_delivery_info_table_checkout_page_for_block($af_wc_daps_rule_settings, $af_wc_daps_product_id_with_largest_value),
				);
			}
		} elseif ($af_wc_daps_rule_id_with_largest_value && $af_wc_daps_largest_minimum_days_rule_level >= $af_wc_daps_largest_minimum_days) {
			$af_wc_daps_post_level_settings = (array) get_post_meta( $af_wc_daps_rule_id_with_largest_value, 'af_wc_daps_rule_settings', true );
			if ( 'expected_date' == $af_wc_daps_post_level_settings['af_wc_daps_rule_delivery_selection_mode'] ) {
				$af_wc_daps_rule_minimum_delivery_days      = isset( $af_wc_daps_post_level_settings['af_wc_daps_rule_minimum_delivery_days'] ) ? $af_wc_daps_post_level_settings['af_wc_daps_rule_minimum_delivery_days'] : '';
				$af_wc_daps_rule_maximum_delivery_days      = isset( $af_wc_daps_post_level_settings['af_wc_daps_rule_maximum_delivery_days'] ) ? $af_wc_daps_post_level_settings['af_wc_daps_rule_maximum_delivery_days'] : '';
				$af_wc_daps_rule_expected_date_message_text = isset( $af_wc_daps_post_level_settings['af_wc_daps_rule_expected_date_message_text'] ) ? $af_wc_daps_post_level_settings['af_wc_daps_rule_expected_date_message_text'] : '';

				return array(
					'selection_mode' => 'expected_date',
					'rule_data'      =>  af_wc_daps_expected_date_info_checkout_page_for_block( $af_wc_daps_rule_minimum_delivery_days, $af_wc_daps_rule_maximum_delivery_days , $af_wc_daps_rule_expected_date_message_text),
				);
			}

			wp_localize_script(
			'af-wc-daps-front-script',
			'af_wc_daps_checkout_page_rule_settings',
			$af_wc_daps_post_level_settings
			);
			
			return array(
				'selection_mode' => 'calendar',
				'rule_data'      =>  af_wc_daps_delivery_info_table_checkout_page_for_block($af_wc_daps_post_level_settings, $af_wc_daps_rule_id_with_largest_value),
			);
			

		}
	}
}
